@@ -29,7 +29,6 @@
 
 import binascii
 import os
-import stat
 import string
 import sys
 
@@ -63,7 +62,7 @@ PARENTDIR = os.path.split(os.path.dirname(progpath))[0] # '../'
 # If the file /etc/tmdarc exists, read it before ~/.tmdarc.
 # Make site-wide configuration changes to this file.
 GLOBAL_TMDARC = '/etc/tmdarc'
-if os.path.isfile(GLOBAL_TMDARC):
+if os.path.exists(GLOBAL_TMDARC):
     try:
         execfile(GLOBAL_TMDARC)
     except:
@@ -74,36 +73,41 @@ if os.path.isfile(GLOBAL_TMDARC):
 TMDARC = os.environ.get('TMDARC')
 if not TMDARC:
     TMDARC = os.path.expanduser('~/.tmdarc')
-    if not os.path.isfile(TMDARC):
+    if not os.path.exists(TMDARC):
         TMDARC = os.path.expanduser('~/.tmda/config')
 
+# CONFIG_EXEC
+# If set to 0 in GLOBAL_TMDARC, the user's TMDARC file will be parsed
+# using ConfigParser, otherwise it will evaluated as a sequence of
+# Python statements using execfile().
+# Default is 1 (execfile())
+if not vars().has_key('CONFIG_EXEC'):
+    CONFIG_EXEC = 1
+
 # Read-in the user's configuration file.
 if os.path.exists(TMDARC):
-    execfile(TMDARC)
-else:
-    raise Errors.ConfigError, "Can't open configuration file: " + TMDARC
-
-# Check for proper file permissions before proceeding.
-statinfo = os.stat(TMDARC)
-permbits = stat.S_IMODE(statinfo[stat.ST_MODE])
-mode = int(oct(permbits))
-
-# ALLOW_MODE_640
-# Set this variable to 1 if you want to allow mode 640 .tmdarc files.
-# Default is 0 (turned off)
-if not vars().has_key('ALLOW_MODE_640'):
-    ALLOW_MODE_640 = 0
-
-if ALLOW_MODE_640 and mode in (400, 600, 640):
-    pass
-elif mode not in (400, 600):
-    raise Errors.ConfigError, TMDARC + " must be chmod 400 or 600!"
+    if CONFIG_EXEC:
+        execfile(TMDARC)
+    else:
+        import ConfigParser
+        cf = ConfigParser.ConfigParser()
+        cf.read(TMDARC)
+        cf_section = 'TMDA_CONFIG'
+        for option in cf.options(cf_section):
+            value = cf.get(cf_section, option)
+            option = option.upper()
+            # Translate options into variables.
+            if value in list(string.digits):
+                # make sure integer values don't get turned into strings
+                exec('%s = %s' % (option, value))
+            else:
+                exec('%s = "%s"' % (option, value))
 
 
 import Util
 import Version
 
-TMDA_HOMEPAGE = "(http://tmda.sf.net/)"
+TMDA_HOMEPAGE = "(http://tmda.net/)"
 TMDA_VERSION = Version.TMDA
 
 ############################
@@ -127,18 +131,34 @@ if not vars().has_key('DATADIR'):
 if not vars().has_key('MAIL_TRANSFER_AGENT'):
     MAIL_TRANSFER_AGENT = "qmail"
 
-# LOCAL_DELIVERY_AGENT
-# The full path to the program used to deliver a sucessful message to
-# your mailbox.  Only necessary if you are NOT running qmail!
-# Tested LDAs include maildrop and procmail.
-# This variable may also contain arguments which will be passed to the command.
-# e.g, LOCAL_DELIVERY_AGENT = "/usr/bin/procmail -p ~/.procmailrc"
-# No default
-if not vars().has_key('LOCAL_DELIVERY_AGENT'):
-    LOCAL_DELIVERY_AGENT = None
-if MAIL_TRANSFER_AGENT != 'qmail' and not LOCAL_DELIVERY_AGENT:
-    raise Errors.ConfigError, \
-          "non-qmail must define LOCAL_DELIVERY_AGENT in " + TMDARC
+# DELIVERY
+# The default delivery instruction for successful messages, or TMDA's
+# final delivery location.  Only required if you are NOT running
+# qmail.  The default value for qmail users is "_qok_" which means
+# exit(0) and proceed to the next instruction in the dot-qmail file.
+#
+# Delivery to qmail-style Maildirs, mboxrd-format mboxes, programs
+# (pipe), and forward to an e-mail addresses are supported.
+#
+# Acceptable syntax and restrictions for delivery instructions are
+# discussed in the ``action'' section of the TMDA Filter Specification
+# (config-filter.html).  Please read this documentation.
+#
+# Examples:
+# DELIVERY = "~/Maildir/"
+# DELIVERY = "~/Mailbox"
+# DELIVERY = "/var/mail/jasonrm"
+# DELIVERY = "|/usr/bin/maildrop"
+# DELIVERY = "|/usr/bin/procmail -f $SENDER ~/.procmailrc-tmda"
+# DELIVERY = "me@new.job.com"
+#
+# No default for non-qmail users.
+if not vars().has_key('DELIVERY'):
+    if MAIL_TRANSFER_AGENT == 'qmail':
+        DELIVERY = "_qok_"
+    else:
+        raise Errors.ConfigError, \
+              "non-qmail users must define DELIVERY in " + TMDARC
 
 # RECIPIENT_DELIMITER
 # A single character which specifies the separator between user names
@@ -149,14 +169,97 @@ if MAIL_TRANSFER_AGENT != 'qmail' and not LOCAL_DELIVERY_AGENT:
 if not vars().has_key('RECIPIENT_DELIMITER'):
     RECIPIENT_DELIMITER = "-"
 
-# SENDMAIL
+# ALLOW_MODE_640
+# Set this variable to 1 if you want to allow a mode 640 CRYPT_KEY_FILE.
+# Default is 0 (turned off)
+if not vars().has_key('ALLOW_MODE_640'):
+    ALLOW_MODE_640 = 0
+
+# CRYPT_KEY_FILE
+# File which contains your unique TMDA secret key generated by the
+# `tmda-keygen' program.  The key should be unquoted in the file.
+# This file must be chmod 400 or 600, unless ALLOW_MODE_640 is on.
+# Default is ~/.tmda/crypt_key
+if not vars().has_key('CRYPT_KEY_FILE'):
+    CRYPT_KEY_FILE = os.path.join(DATADIR, 'crypt_key')
+if os.path.exists(CRYPT_KEY_FILE):
+    crypt_key_filemode = Util.getfilemode(CRYPT_KEY_FILE)
+    if crypt_key_filemode not in (400, 600):
+        if ALLOW_MODE_640 and crypt_key_filemode == 640:
+            pass
+        else:
+            raise Errors.ConfigError, \
+                  CRYPT_KEY_FILE + " must be chmod 400 or 600!"
+else:
+    raise Errors.ConfigError, "Can't find key file: " + CRYPT_KEY_FILE
+
+# Read key from CRYPT_KEY_FILE, and then convert it from hex back into
+# raw binary.  Hex has only 4 bits of entropy per byte as opposed to 8.
+CRYPT_KEY = binascii.unhexlify(open(CRYPT_KEY_FILE).read().strip())
+
+# OUTGOINGMAIL
+# Final delivery method for all outgoing mail (server and client).
+# Possible values include:
+# 
+# "smtp"
+#    Deliver messages by handing them off to an SMTP server (i.e,
+#    smarthost).  Be sure that the "SMTPHOST" variable (see below) is
+#    set correctly.
+#
+# "sendmail"
+#    Deliver messages via the command line interface to the sendmail
+#    program (/usr/sbin/sendmail).  Use at your own risk.  "smtp" is
+#    highly recommended.
+#
+#    SECURITY WARNING: The 'sendmail' method is not secure.  Because
+#    this method uses popen(), it goes through the shell.  It does not
+#    scan the arguments for potential exploits and so it should be
+#    considered unsafe.  For performance reasons, it's not recommended
+#    either -- use the 'smtp' method instead, even if
+#    MAIL_TRANSFER_AGENT is "sendmail".
+#
+# Default is "smtp"
+if not vars().has_key('OUTGOINGMAIL'):
+    OUTGOINGMAIL = "smtp"
+
+# SMTPHOST
+# SMTP host and optional port, when OUTGOINGMAIL is "smtp".
+# If the hostname or IP address ends with a colon (":") followed by a
+# number, that suffix will be stripped off and the number interpreted
+# as the port number to use.  Otherwise, the standard SMTP port (25)
+# will be used.
+#
+# Examples:
+#
+# SMTPHOST = "localhost"
+# SMTPHOST = "172.16.78.20:2525"
+# SMTPHOST = "mailhost.company.com"
+# SMTPHOST = "mailhost.company.com:1234"
+#
+# Default is "localhost" (port 25 on the local host)
+if not vars().has_key('SMTPHOST') and OUTGOINGMAIL == 'smtp':
+    SMTPHOST = "localhost"
+
+# SMTP_MAX_SESSIONS_PER_CONNECTION
+# An integer specifying a ceiling on the number of SMTP sessions to
+# perform on a single socket connection, when OUTGOINGMAIL is
+# "smtp".  Some MTAs have limits.  Set this to 0 to do as many
+# as we like (i.e. your MTA has no limits).  Set this to some number
+# great than 0 and TMDA will close the SMTP connection and re-open it
+# after this number of consecutive sessions.
+# Default is 0
+if not vars().has_key('SMTP_MAX_SESSIONS_PER_CONNECTION') and \
+       OUTGOINGMAIL == 'smtp':
+    SMTP_MAX_SESSIONS_PER_CONNECTION = 0
+
+# SENDMAIL_PROGRAM
 # The path to the sendmail program, or sendmail compatibility
-# interface.  Defaults to one of the two standard locations, but you
-# can override it in case it is installed elsewhere.
-if not vars().has_key('SENDMAIL'):
+# interface when OUTGOINGMAIL is "sendmail".
+# Defaults to one of the two standard locations.
+if not vars().has_key('SENDMAIL_PROGRAM') and OUTGOINGMAIL == 'sendmail':
     for sendmail in ('/usr/sbin/sendmail', '/usr/lib/sendmail'):
         if os.path.exists(sendmail):
-            SENDMAIL = sendmail
+            SENDMAIL_PROGRAM = sendmail
             break
 
 # USEVIRTUALDOMAINS
@@ -175,18 +278,19 @@ if not vars().has_key('VIRTUALDOMAINS'):
     VIRTUALDOMAINS = "/var/qmail/control/virtualdomains"
 
 # BOUNCE_ENV_SENDER
-# The envelope sender of the bounce message.
+# The envelope sender of a bounce message.
 #
 # Example:
 # BOUNCE_ENV_SENDER = "devnull@domain.dom"
 #
 # Default is an empty envelope sender <>.
 if not vars().has_key('BOUNCE_ENV_SENDER'):
-    # Exim and Sendmail don't like -f ''
-    if MAIL_TRANSFER_AGENT in ('exim', 'sendmail'):
-        BOUNCE_ENV_SENDER = '<>'
-    else:
+    if OUTGOINGMAIL == 'sendmail' and \
+           MAIL_TRANSFER_AGENT in ('qmail', 'postfix'):
+        # qmail/Postfix's /usr/sbin/sendmail doesn't like -f '<>'
         BOUNCE_ENV_SENDER = ''
+    else:
+        BOUNCE_ENV_SENDER = '<>'
 
 # BOUNCE_TEXT_FILTER_INCOMING
 # Text for the failure notice returned to the sender when a 'bounce'
@@ -208,6 +312,38 @@ if not vars().has_key('BOUNCE_TEXT_INVALID_CONFIRMATION'):
 # located.  Set to None to disable notification.
 if not vars().has_key('BOUNCE_TEXT_NONEXISTENT_PENDING'):
     BOUNCE_TEXT_NONEXISTENT_PENDING = """Your original message could not be located."""
+
+# BOUNCE_TEXT_FAIL_DATED
+# Text for the failure notice returned to the sender when a
+# dated address does not verify with a correct HMAC.  Set to None 
+# to disable notification.  Setting to None will cause the same behavior
+# as setting ACTION_FAIL_DATED to drop
+if not vars().has_key('BOUNCE_TEXT_FAIL_DATED'):
+    BOUNCE_TEXT_FAIL_DATED = """You have sent email to an invalid address."""
+
+# BOUNCE_TEXT_EXPIRED_DATED
+# Text for the failure notice returned to the sender when a
+# dated address is expired.  Set to None to disable notification.  Setting
+# to None will cause the same behavior as setting ACTION_EXPIRED_DATED to
+# drop
+if not vars().has_key('BOUNCE_TEXT_EXPIRED_DATED'):
+    BOUNCE_TEXT_EXPIRED_DATED = """You have sent email to an expired address."""
+
+# BOUNCE_TEXT_FAIL_SENDER
+# Text for the failure notice returned to the sender when a
+# sender address does not verify with a correct HMAC.  Set to None
+# to disable notification.  Setting to None will cause the same behavior
+# as setting ACTION_FAIL_SENDER to drop
+if not vars().has_key('BOUNCE_TEXT_FAIL_SENDER'):
+    BOUNCE_TEXT_FAIL_SENDER = """You have sent email to an address you are not authorized to use."""
+
+# BOUNCE_TEXT_FAIL_KEYWORD
+# Text for the failure notice returned to the sender when a
+# keyword address does not verify with a correct HMAC. Set to None
+# to disable notification.  Setting to None will cause the same behavior
+# as setting ACTION_FAIL_KEYWORD to drop.
+if not vars().has_key('BOUNCE_TEXT_FAIL_KEYWORD'):
+    BOUNCE_TEXT_FAIL_KEYWORD = """You have sent email to an invalid address."""
         
 # BARE_APPEND
 # Filename to which a recipient's e-mail address should be
@@ -337,17 +473,6 @@ if not vars().has_key('DATED_TEMPLATE_VARS'):
 if not vars().has_key('SENDER_TEMPLATE_VARS'):
     SENDER_TEMPLATE_VARS = 0
 
-# CRYPT_KEY
-# Your encryption key should be unique and kept secret.
-# Use the included "tmda-keygen" program to generate your key.
-# No default.
-if not vars().has_key('CRYPT_KEY'):
-    raise Errors.ConfigError, "CRYPT_KEY not defined in " + TMDARC
-else:
-    # Convert key from hex back into raw binary.
-    # Hex has only 4 bits of entropy per byte as opposed to 8.
-    CRYPT_KEY = binascii.unhexlify(CRYPT_KEY)
-    
 # FILTER_INCOMING
 # Filter file which controls how incoming messages are tagged.
 # Look for the filter-file in the environment first.
@@ -412,6 +537,91 @@ if not vars().has_key('FILTER_DROP_CC'):
 if not vars().has_key('ACTION_INCOMING'):
     ACTION_INCOMING = "confirm"
 
+# ACTION_FAIL_DATED
+# Specifies how incoming messages should be disposed of if they are sent
+# to a dated address that does not properly verify.  
+# Possible values include:
+#
+# "bounce"
+#    bounce the message - uses BOUNCE_TEXT_FAIL_DATED
+#
+# "drop"
+#    silently drop the message
+#
+# "ok"
+#    deliver the message
+#
+# "confirm"
+#    request confirmation for the message
+#
+# Default is confirm
+if not vars().has_key('ACTION_FAIL_DATED'):
+    ACTION_FAIL_DATED = "confirm"
+
+# ACTION_EXPIRED_DATED
+# Specifies how incoming messages should be disposed of if they are sent
+# to a dated address that does not properly verify.  
+# Possible values include:
+#
+# "bounce"
+#    bounce the message - uses BOUNCE_TEXT_EXPIRED_DATED
+#
+# "drop"
+#    silently drop the message
+#
+# "ok"
+#    deliver the message
+#
+# "confirm"
+#    request confirmation for the message
+#
+# Default is confirm
+if not vars().has_key('ACTION_EXPIRED_DATED'):
+    ACTION_EXPIRED_DATED = "confirm"
+
+# ACTION_FAIL_SENDER
+# Specifies how incoming messages should be disposed of if they are sent 
+# to a sender address, but were not sent from the correct sender, and 
+# fail to verify.
+# Possible values include:
+#
+# "bounce"
+#    bounce the message - uses BOUNCE_TEXT_FAIL_SENDER
+#
+# "drop"
+#    silently drop the message
+#
+# "ok"
+#    deliver the message
+#
+# "confirm"
+#    request confirmation for the message
+#
+# Default is confirm
+if not vars().has_key('ACTION_FAIL_SENDER'):
+    ACTION_FAIL_SENDER = "confirm"
+
+# ACTION_FAIL_KEYWORD
+# Specifies how incoming messages should be disposed of if they are
+# sent to a keyword address that fails to verify.
+# Possible values include:
+#
+# "bounce"
+#    bounce the message - uses BOUNCE_TEXT_FAIL_KEYWORD
+#
+# "drop"
+#    silently drop the message
+#
+# "ok"
+#    deliver the message
+#
+# "confirm"
+#    request confirmation for the message
+#
+# Default is confirm
+if not vars().has_key('ACTION_FAIL_KEYWORD'):
+    ACTION_FAIL_KEYWORD = "confirm"
+
 # ACTION_OUTGOING
 # Specifies how outgoing messages should be tagged by default if there
 # are no matches in FILTER_OUTGOING and no X-TMDA header.
@@ -518,22 +728,34 @@ if not vars().has_key('HOSTNAME'):
 # Filename which uncaught exceptions should be written to.
 #
 # Example:
-# LOGFILE_DEBUG = "/path/to/tmda_debug.log"
+# LOGFILE_DEBUG = "/path/to/logs/tmda.debug"
 #
 # No default.
 if not vars().has_key('LOGFILE_DEBUG'):
     LOGFILE_DEBUG = None
     
 # LOGFILE_INCOMING
-# Filename which delivery summaries should be written to.
+# Filename which incoming delivery (i.e, tmda-filter) summaries should
+# be written to.
 #
 # Example:
-# LOGFILE_INCOMING = "/path/to/tmda_incoming.log"
+# LOGFILE_INCOMING = "/path/to/logs/tmda.in"
 #
 # No default.
 if not vars().has_key('LOGFILE_INCOMING'):
     LOGFILE_INCOMING = None
 
+# LOGFILE_OUTGOING
+# Filename which outgoing message (i.e, tmda-sendmail) summaries
+# should be written to.
+#
+# Example:
+# LOGFILE_OUTGOING = "/path/to/logs/tmda.out"
+#
+# No default.
+if not vars().has_key('LOGFILE_OUTGOING'):
+    LOGFILE_OUTGOING = None
+
 # LOCALDATE
 # Set this variable to 0 if you want TMDA to generate ``Date:''
 # headers in UTC instead of the local time zone.
@@ -595,6 +817,17 @@ if not vars().has_key('PENDING_CACHE'):
 if not vars().has_key('PENDING_CACHE_LEN'):
     PENDING_CACHE_LEN = 5000
 
+# PENDING_BLACKLIST_APPEND
+# Filename to which a sender's e-mail address should be appended
+# when a message is "blacklisted" by tmda-pending.
+#
+# Example:
+# PENDING_BLACKLIST_APPEND = "/full/path/to/blacklist"
+#
+# No default
+if not vars().has_key('PENDING_BLACKLIST_APPEND'):
+    PENDING_BLACKLIST_APPEND = None
+
 # PENDING_DELETE_APPEND
 # Filename to which a sender's e-mail address should be automatically
 # appended when a message is "deleted" by tmda-pending.
@@ -617,6 +850,58 @@ if not vars().has_key('PENDING_DELETE_APPEND'):
 if not vars().has_key('PENDING_RELEASE_APPEND'):
     PENDING_RELEASE_APPEND = None
 
+# PENDING_WHITELIST_APPEND
+# Filename to which a sender's e-mail address should be appended
+# when a message is "whitelisted" by tmda-pending.
+#
+# Example:
+# PENDING_WHITELIST_APPEND = "/full/path/to/whitelist"
+#
+# No default
+if not vars().has_key('PENDING_WHITELIST_APPEND'):
+    PENDING_WHITELIST_APPEND = None
+
+# ADDED_HEADERS_CLIENT
+# A Python dictionary containing one or more header:value string pairs
+# that should be added to _all_ outgoing client-side messages (i.e,
+# messages sent with tmda-sendmail) prior to injection.  Listed
+# headers and their values are case-sensitive.
+#
+# As the examples below illustrate, the full power of Python is
+# available to create these headers -- just make sure both the header
+# and its value end up as strings.
+#
+# Examples:
+#
+# ADDED_HEADERS_CLIENT = {"X-Fact" : "Father Hennepin Discovered Niagra Falls"}
+# import time, os, random
+# ADDED_HEADERS_CLIENT = {
+#     "X-Localtime" : time.asctime(),
+#     "X-Favorite-Author" : "James Joyce",
+#     "X-OperatingSystem" : os.uname()[0],
+#     "Organization" : os.environ.get("ORGANIZATION"),
+#     "X-Uptime" : os.popen("/usr/bin/uptime").read().strip(),
+#     "X-Now-Reading" : random.choice(open
+#                                     ("/home/jasonrm/.now-reading").
+#                                     readlines()).strip()
+#     }
+#
+# No default
+if not vars().has_key('ADDED_HEADERS_CLIENT'):
+    ADDED_HEADERS_CLIENT = None
+
+# ADDED_HEADERS_SERVER
+# A Python dictionary containing one or more header:value string pairs
+# that should be added to _all_ outgoing server-side messages (i.e,
+# messages sent with tmda-filter) prior to injection.  Listed
+# headers and their values are case-sensitive.
+#
+# See ADDED_HEADERS_CLIENT (above) for some examples.
+#
+# No default
+if not vars().has_key('ADDED_HEADERS_SERVER'):
+    ADDED_HEADERS_SERVER = None
+
 # PURGED_HEADERS
 # A list containing one or more message headers that should be removed
 # from outgoing client-side messages (i.e, messages sent with
@@ -624,14 +909,16 @@ if not vars().has_key('PENDING_RELEASE_APPEND'):
 # case-insensitive, and the purging will only be attempted if the
 # header actually exists.
 #
+# NOTE: PURGED_HEADERS is run _after_ ADDED_HEADERS (see above).
+#
 # Examples:
 #
-# PURGED_HEADERS = ["x-mailer"]
-# PURGED_HEADERS = ["x-mailer", "user-agent"]
+# PURGED_HEADERS = ["bcc", "resent-bcc", "x-mailer"]
+# PURGED_HEADERS = ["bcc", "resent-bcc", "x-mailer", "user-agent"]
 #
-# No default
+# Default is "Bcc:" and "Resent-Bcc"
 if not vars().has_key('PURGED_HEADERS'):
-    PURGED_HEADERS = None
+    PURGED_HEADERS = ["bcc", "resent-bcc"]
 
 # RECIPIENT_HEADER
 # A string containing the name of a header (case-insensitive) whose
@@ -662,6 +949,59 @@ if not vars().has_key('PURGED_HEADERS'):
 if not vars().has_key('RECIPIENT_HEADER'):
     RECIPIENT_HEADER = None
 
+# TAGS_CONFIRM
+# A list of one or more strings used to identify a confirmation
+# address.  The first element in the list will be used when
+# generating a new confirmation address.
+#
+# Example:
+#
+# TAGS_CONFIRM = ['confirm', 'c']
+#
+# Default is 'confirm'
+if not vars().has_key('TAGS_CONFIRM'):
+    TAGS_CONFIRM = ['confirm']
+
+# TAGS_DATED
+# A list of one or more strings used to identify a dated address.  The
+# first element in the list will be used by `tmda-address' when
+# generating a new dated address.
+#
+# Example:
+#
+# TAGS_DATED = ['dated', 'd', 'exp', 'expires']
+#
+# Default is 'dated'
+if not vars().has_key('TAGS_DATED'):
+    TAGS_DATED = ['dated']
+
+# TAGS_KEYWORD
+# A list of one or more strings used to identify a keyword address.
+# The first element in the list will be used by `tmda-address' when
+# generating a new keyword address.
+#
+# Example:
+#
+# TAGS_KEYWORD = ['keyword', 'key', 'kw']
+#
+# Default is 'keyword'
+#
+if not vars().has_key('TAGS_KEYWORD'):
+    TAGS_KEYWORD = ['keyword']
+
+# TAGS_SENDER
+# A list of one or more strings to identify a sender address.  The
+# first element in the list will be used by `tmda-address' when
+# generating a new sender address.
+#
+# Example:
+#
+# TAGS_SENDER = ['sender', 's']
+#
+# Default is 'sender'
+if not vars().has_key('TAGS_SENDER'):
+    TAGS_SENDER = ['sender']
+
 # TERSE_SUMMARY_HEADERS
 # A list containing one or more message headers that should be
 # displayed by tmda-pending's `--terse option'.  Listed headers are
@@ -694,7 +1034,31 @@ if not vars().has_key('TIMEOUT'):
 # Defaults to your UNIX username.
 if not vars().has_key('USERNAME'):
     USERNAME = Util.getusername()
-    
+
+# X_TMDA_IN_SUBJECT
+# With this variable set to 1, tmda-inject will parse the Subject
+# header looking for `X-TMDA' actions, and then remove them before
+# sending the message.  This is useful for users that desire the
+# `X-TMDA' override behavior, but don't use an MUA which easily allows
+# addition of arbitrary headers (e.g, Outlook).
+#
+# The Subject header should contain `X-TMDA' followed by whitespace
+# followed by the desired action followed by the real subject.  Case
+# is insensitive.
+#
+# Examples:
+#
+# Subject: X-TMDA dated Re: You're fired!
+# Subject:     X-TMDA dated=5M Re: You're fired!
+# Subject: X-TMDA   sender      Re: You're fired!
+#
+# In all cases, the resulting subject will simply be:
+# Subject: Re: You're fired!
+#
+# Default is 0 (turned off)
+if not vars().has_key('X_TMDA_IN_SUBJECT'):
+    X_TMDA_IN_SUBJECT = 0
+
 ###################################
 # END of user configurable settings
 ###################################
