@@ -27,12 +27,17 @@ import fileinput
 import fnmatch
 import os
 import popen2
+import pwd
 import re
+import socket
+import stat
 import string
 import sys
 import tempfile
 import time
 
+import Errors
+
 
 EMPTYSTRING = ''
 NL = '\n'
@@ -44,7 +49,6 @@ def gethostname():
     hostname = os.environ.get('QMAILHOST') or \
                os.environ.get('MAILHOST')
     if not hostname:
-        import socket
         hostname = socket.getfqdn()
     return hostname
 
@@ -54,7 +58,6 @@ def getfullname():
                os.environ.get('NAME') or \
                os.environ.get('MAILNAME')
     if not fullname:
-        import pwd
         fullname = pwd.getpwuid(os.getuid())[4]
     if not fullname:
         fullname = ''
@@ -66,13 +69,53 @@ def getusername():
                os.environ.get('USER') or \
                os.environ.get('LOGNAME')
     if not username:
-        import pwd
         username = pwd.getpwuid(os.getuid())[0]
     if not username:
         username = '<unknown>'
     return username
 
 
+def getuid(username):
+    """Return username's numerical user ID."""
+    return pwd.getpwnam(username)[2]
+
+
+def getgid(username):
+    """Return username's numerical group ID."""
+    return pwd.getpwnam(username)[3]
+
+
+def gethomedir(username):
+    """Return the home directory of username."""
+    return pwd.getpwnam(username)[5]
+
+
+def getgrouplist(username):
+    """Read through the group file and calculate the group access
+    list for the specified user.  Return a list of group ids."""
+    import grp
+    # calculate the group access list
+    gids = [i[2] for i in grp.getgrall() if username in i[-1]]
+    # include the base gid
+    gids.insert(0, getgid(username))
+    return gids
+
+
+def getfilemode(path):
+    """Return the octal number of the bit pattern for the file
+    permissions on path."""
+    statinfo = os.stat(path)
+    permbits = stat.S_IMODE(statinfo[stat.ST_MODE])
+    mode = int(oct(permbits))
+    return mode
+
+
+def getfileuid(path):
+    """Return the numerical UID of the user owning the file in path."""
+    statinfo = os.stat(path)
+    return statinfo[stat.ST_UID]
+
+
 def seconds(timeout):
     """Translate the defined timeout interval into seconds."""
     match = re.match("^([0-9]+)([YMwdhms])$", timeout)
@@ -158,13 +201,15 @@ def make_msgid(timesecs=None, pid=None):
 
 
 def make_date(timesecs=None):
-    """Return an RFC 2822 compliant Date: string.
+    """Return an RFC 2822 compliant Date: string.  e.g,
+
+    Thu, 16 May 2002 04:23:10 +1200 (NZST)
     
     timesecs is optional, and if not given, the current time is used.
 
-    JRM: Once the email mod is included within TMDA, we can nuke this
-    function and use email.Utils.formatdate.
-
+    JRM: Once the email module is included within TMDA, we can use
+    utilize email.Utils.formatdate.
+    
     Based on code from Python's email package
     <URL:http://www.python.org/doc/current/lib/module-email.html>
     Copyright (C) 2001,2002 Python Software Foundation.
@@ -189,15 +234,18 @@ def make_date(timesecs=None):
         else:
             sign = '+'
         zone = '%s%02d%02d' % (sign, hours, minutes / 60)
+        tzname = time.tzname[now[-1]]
     else:
         now = time.gmtime(timesecs)
         # Timezone offset is always -0000
         zone = '-0000'
-    return '%02d %s %04d %02d:%02d:%02d %s' % (
-        now[2], ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
-                 'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][now[1] - 1],
-        now[0], now[3], now[4], now[5],
-        zone)
+        tzname = 'UTC'
+    return '%s, %02d %s %04d %02d:%02d:%02d %s (%s)' % (
+        ['Mon', 'Tue', 'Wed', 'Thu', 'Fri', 'Sat', 'Sun'][now[6]],
+        now[2],
+        ['Jan', 'Feb', 'Mar', 'Apr', 'May', 'Jun',
+         'Jul', 'Aug', 'Sep', 'Oct', 'Nov', 'Dec'][now[1] - 1],
+        now[0], now[3], now[4], now[5], zone, tzname)
 
 
 def formataddr(pair):
@@ -262,6 +310,8 @@ def pipecmd(command, *strings):
 
     *strings are optional pieces of data to write to command.
 
+    return_status will just return the exit status of the command..
+    
     Based on code from getmail
     <URL:http://www.qcc.sk.ca/~charlesc/software/getmail-2.0/>
     Copyright (C) 2001 Charles Cazabon, and licensed under the GNU
@@ -355,26 +405,32 @@ def pager(file):
     os.spawnvp(os.P_WAIT, pager_list[0], pager_list)
 
 
-def sendmail(headers, body, recip, return_path=None):
-    """Send e-mail by opening a pipe to the sendmail program.
-
-    headers can be either a rfc822.Message instance, or a set of
+def sendmail(headers, body, envrecip, envsender):
+    """Send e-mail via direct SMTP, or by opening a pipe to the
+    sendmail program.
+    
+    headers can be either an rfc822.Message instance, or a set of
     rfc822 compatible message headers as a string.
 
     body is the message body content as a string.
 
-    recip is the recipient e-mail address.
+    envrecip is the envelope recipient address.
 
-    return_path is an optional e-mail address which the envelope
-    sender address of the message will be set to.
+    envsender is the envelope sender address.
     """
     import Defaults
-    if return_path is not None:
-        env_sender = "-f '%s'" % (return_path)
+    if Defaults.OUTGOINGMAIL == 'smtp':
+        import SMTP
+        server = SMTP.Connection()
+        server.sendmail(envsender, envrecip, (str(headers) + '\n' + body))
+        server.quit()
+    elif Defaults.OUTGOINGMAIL == 'sendmail':
+        cmd = "%s -f '%s' -- '%s'" % (Defaults.SENDMAIL_PROGRAM,
+                                      envsender, envrecip)
+        pipecmd(cmd, str(headers), '\n', body)
     else:
-        env_sender = ''
-    cmd = "%s %s '%s'" % (Defaults.SENDMAIL, env_sender, recip)
-    pipecmd(cmd, str(headers), '\n', body)
+        raise Errors.ConfigError, \
+              "Invalid OUTGOINGMAIL method: " + Defaults.OUTGOINGMAIL
 
 
 def build_cdb(filename):
